package main;

import java.io.IOException;

import org.hipparchus.geometry.euclidean.threed.Vector3D;
import org.hipparchus.ode.nonstiff.AdaptiveStepsizeIntegrator;
import org.hipparchus.ode.nonstiff.DormandPrince853Integrator;
import org.orekit.attitudes.AttitudeProvider;
import org.orekit.attitudes.BodyCenterPointing;
import org.orekit.attitudes.YawSteering;
import org.orekit.bodies.BodyShape;
import org.orekit.bodies.CelestialBodyFactory;
import org.orekit.bodies.Ellipsoid;
import org.orekit.bodies.OneAxisEllipsoid;
import org.orekit.errors.OrekitException;
import org.orekit.forces.ForceModel;
import org.orekit.forces.gravity.HolmesFeatherstoneAttractionModel;
import org.orekit.forces.gravity.potential.GravityFieldFactory;
import org.orekit.forces.gravity.potential.NormalizedSphericalHarmonicsProvider;
import org.orekit.frames.Frame;
import org.orekit.frames.FramesFactory;
import org.orekit.orbits.KeplerianOrbit;
import org.orekit.orbits.Orbit;
import org.orekit.orbits.OrbitType;
import org.orekit.orbits.PositionAngle;
import org.orekit.propagation.SpacecraftState;
import org.orekit.propagation.events.ElevationDetector;
import org.orekit.propagation.events.handlers.ContinueOnEvent;
import org.orekit.propagation.numerical.NumericalPropagator;
import org.orekit.time.AbsoluteDate;
import org.orekit.utils.Constants;
import org.orekit.utils.IERSConventions;
import org.orekit.utils.PVCoordinatesProvider;

public class mainProp {

	private static final double mu = Constants.WGS84_EARTH_MU;
	double a = 6878000;
	double e = 0.0000000001;
	double i = 0.0000000001;
	double raan = 0.0;
	double omega = 0;
	double trueAnomaly = 0;
	Frame frameinertial;
	Frame orbitFrame;
	int stepsize = 5;
	AbsoluteDate initialDate;

	private Orbit orbit;
	private final OrbitType propagationType;

	BodyShape earthBody;
	PVCoordinatesProvider sun;
	Vector3D vector;

	BodyCenterPointing nadirLawZ;
	private final AttitudeProvider attProv;

	private SpacecraftState state;

	Sequencer sequencer;

	groundstation gs;
	final ElevationDetector gsVis;
	
	double minStep = 0.001;
	double maxStep = 100;
	double positionTol = 10;
	double[][] tolerance;

	NumericalPropagator propagator;
	ForceModel holmesFeatherstone;
	
	public mainProp(AbsoluteDate initial) throws OrekitException {
		this.initialDate = initial;
		frameinertial = FramesFactory.getITRF(IERSConventions.IERS_2010, true);
		orbitFrame = FramesFactory.getEME2000();
		orbit = new KeplerianOrbit(a, e, i, omega, raan, trueAnomaly,PositionAngle.TRUE, orbitFrame, initialDate, mu);
		earthBody = new OneAxisEllipsoid(Constants.WGS84_EARTH_EQUATORIAL_RADIUS,
				Constants.WGS84_EARTH_FLATTENING, frameinertial);
		sun = CelestialBodyFactory.getSun();
		vector = new Vector3D(1, 1, 0);
		nadirLawZ = new BodyCenterPointing(orbit.getFrame(), (Ellipsoid) earthBody);
		attProv = new YawSteering(orbitFrame, nadirLawZ, sun, vector.normalize());
		state = new SpacecraftState(orbit, attProv.getAttitude(orbit, orbit.getDate(), orbit.getFrame()));
		sequencer = new Sequencer(attProv);
		gs = new groundstation( 0.3, 3.1415, 0.0, 0.087);
		gsVis = new ElevationDetector(60, 0.001, gs.getFrame());
		propagationType = orbit.getType();
		tolerance = NumericalPropagator.tolerances(positionTol, orbit, propagationType);
	}

	public void startSim() throws OrekitException, IOException {
		gs.initFrame(earthBody);
		//final ElevationDetector gsVis = new ElevationDetector(60, 0.001, gs.getFrame());
		final ElevationDetector gsVis = new ElevationDetector(60, 0.001, gs.getFrame()).withConstantElevation(gs.minEleveation).withHandler(new ContinueOnEvent<ElevationDetector>());;

		//final ElevationDetector gsVisibility = new ElevationDetector(DefaultParameters.ELEV_DETECTOR_MAX_CHECK_INTERVAL,DefaultParameters.ELEV_DETECTOR_CONVERGENCE_THRESHOLD, eachGS.getFrame()).withConstantElevation(eachGS.getMinElevation()).withHandler(new ElevationVisibilityHandler(this,passResolution, eachGS, config.step, initialDate.shiftedBy(duration)));
		
		
		sequencer.targetPoint(gs.getLocation().getNadir(), orbit.getFrame(), gs.getFrame(), earthBody, gsVis);
		createPropagator();
		setPropagator();
		sequencer.custSeq.registerSwitchEvents(propagator);

	}
	
	void createPropagator() throws OrekitException {
		AdaptiveStepsizeIntegrator integrator = new DormandPrince853Integrator(minStep, maxStep, tolerance[0], tolerance[1]);
		propagator = new NumericalPropagator(integrator);
		NormalizedSphericalHarmonicsProvider provider = GravityFieldFactory.getNormalizedProvider(10, 10);
		holmesFeatherstone = new HolmesFeatherstoneAttractionModel(FramesFactory.getITRF(IERSConventions.IERS_2010, true), provider);
	}
	
	void setPropagator() throws IOException, OrekitException {
		gsVis.init(state, initialDate);
		propagator.setOrbitType(propagationType);
		propagator.addForceModel(holmesFeatherstone);
		propagator.setMasterMode(stepsize, new propStepHandler());
		propagator.setInitialState(state);
		propagator.setAttitudeProvider(sequencer.custSeq);
	}
}
